// CloudObject.ts

// cloud-object.ts
let serverURL = `https://dev.fmode.cn/parse`;
if (location.protocol == "http:") {
    serverURL = `http://dev.fmode.cn:1337/parse`;
}

export class CloudObject {
    className: string;
    id: string | undefined = undefined;
    createdAt: any;
    updatedAt: any;
    data: Record<string, any> = {};

    constructor(className: string) {
        this.className = className;
    }

    toPointer() {
        return { "__type": "Pointer", "className": this.className, "objectId": this.id };
    }

    set(json: Record<string, any>) {
        Object.keys(json).forEach(key => {
            if (["objectId", "id", "createdAt", "updatedAt"].includes(key)) {
                return;
            }
            this.data[key] = json[key];
        });
    }

    get(key: string) {
        return this.data[key] || null;
    }

    async save() {
        let method = "POST";
        let url = `${serverURL}/classes/${this.className}`;

        // 更新
        if (this.id) {
            url += `/${this.id}`;
            method = "PUT";
        }

        const body = JSON.stringify(this.data);
        const response = await fetch(url, {
            headers: {
                "Content-Type": "application/json;charset=UTF-8",
                "X-Parse-Application-Id": "dev"
            },
            body: body,
            method: method,
            mode: "cors",
            credentials: "omit"
        });

        const result = await response.json();
        if (result.error) {
            console.error(result.error);
        }
        if (result.objectId) {
            this.id = result.objectId;
            this.createdAt = result.createdAt;
            this.updatedAt = result.updatedAt;
        }
        return this;
    }

    async destroy() {
        if (!this.id) return;
        const response = await fetch(`${serverURL}/classes/${this.className}/${this.id}`, {
            headers: {
                "X-Parse-Application-Id": "dev"
            },
            method: "DELETE",
            mode: "cors",
            credentials: "omit"
        });

        const result = await response.json();
        if (result) {
            this.id = undefined;
            this.createdAt = undefined;
            this.updatedAt = undefined;
            this.data = {};
        }
        return true;
    }
}
// CloudQuery.ts
// cloud-query.ts
export class CloudQuery {
    className: string;
    queryParams: Record<string, any> = { where: {} };

    constructor(className: string) {
        this.className = className;
    }

    include(...fields: string[]) {
        this.queryParams["include"] = fields.join(",");
    }

    greaterThan(key: string, value: any) {
        if (!this.queryParams["where"][key]) this.queryParams["where"][key] = {};
        this.queryParams["where"][key]["$gt"] = value;
    }

    greaterThanAndEqualTo(key: string, value: any) {
        if (!this.queryParams["where"][key]) this.queryParams["where"][key] = {};
        this.queryParams["where"][key]["$gte"] = value;
    }

    lessThan(key: string, value: any) {
        if (!this.queryParams["where"][key]) this.queryParams["where"][key] = {};
        this.queryParams["where"][key]["$lt"] = value;
    }

    lessThanAndEqualTo(key: string, value: any) {
        if (!this.queryParams["where"][key]) this.queryParams["where"][key] = {};
        this.queryParams["where"][key]["$lte"] = value;
    }

    equalTo(key: string, value: any) {
        this.queryParams["where"][key] = value;
    }

    async get(id: string) {
        const url = `${serverURL}/classes/${this.className}/${id}`;

        const response = await fetch(url, {
            headers: {
                "X-Parse-Application-Id": "dev"
            },
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response.json();
        if (json.error) {
            console.error(json.error);
            return null;
        }
        return this.dataToObj(json);
    }

    async find(): Promise<Array<CloudObject>> {
        let url = `${serverURL}/classes/${this.className}?`;

        let queryStr = "";
        Object.keys(this.queryParams).forEach(key => {
            let paramStr = JSON.stringify(this.queryParams[key]);
            if (key === "include") {
                paramStr = this.queryParams[key];
            }
            if (queryStr) {
                url += `&${key}=${encodeURIComponent(paramStr)}`;
            } else {
                url += `${key}=${encodeURIComponent(paramStr)}`;
            }
        });

        const response = await fetch(url, {
            headers: {
                "X-Parse-Application-Id": "dev"
            },
            method: "GET",
            mode: "cors",
            credentials: "omit"
        });

        const json = await response.json();
        let list = json.results || [];
        let objList = list.map((item: any) => this.dataToObj(item));
        return objList;
    }

    dataToObj(exists: any): CloudObject {
        let existsObject = new CloudObject(this.className);
        Object.keys(exists).forEach(key => {
            if (exists[key]?.__type === "Pointer") {
                exists[key] = new CloudObject(exists[key].className);
                exists[key].id = exists[key].objectId;
            }
        });
        existsObject.set(exists);
        existsObject.id = exists.objectId;
        existsObject.createdAt = exists.createdAt;
        existsObject.updatedAt = exists.updatedAt;
        return existsObject;
    }
}